package 
{
import com.diadraw.extensions.GameplayRecorder;
import com.diadraw.extensions.GameplayRecorderEvent;
import com.diadraw.extensions.VideoSettings;

import flash.display.BitmapData;
import flash.display.StageAlign;
import flash.display.StageScaleMode;
import flash.display3D.Context3D;
import flash.filesystem.File;
import flash.geom.Point;
import flash.geom.Rectangle;
import flash.utils.getTimer;

import starling.animation.Transitions;
import starling.core.RenderSupport;
import starling.core.Starling;
import starling.display.Image;
import starling.display.Quad;
import starling.display.Sprite;
import starling.events.EnterFrameEvent;
import starling.events.Event;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;
import starling.text.TextField;
import starling.utils.deg2rad;
	
	public class Game extends Sprite
	{	
		private var quads:Vector.<Quad>;
		private var infoLabel:TextField;

		// This won't be added to the stage. This is where you draw your video frames.
		private var invisibleSprite : Sprite;
		
		public function Game()
		{	
			addEventListener(Event.ADDED_TO_STAGE, onAdded);
		}
		
		private function onAdded ( e:Event ):void
		{
			invisibleSprite = new Sprite();
			
			
			quads = new Vector.<Quad>(5, true);
			
			//creating some quads with different colours
			//position the first four of them them in the four courners of the stage
			//and the last in the center of the screen
			quads[0] = new Quad(200, 200, 0xAA0000);
			quads[0].x = 0; quads[0].y=0;
			//set rotation center to be in the center of the quad
			quads[0].pivotX = 100; quads[0].pivotY = 100;
			invisibleSprite.addChild ( quads[0] );
			
			quads[1] = new Quad(200, 200, 0x00FF00);
			quads[1].x = stage.stageWidth; quads[1].y = 0;
			//set rotation center to be in the center of the quad
			quads[1].pivotX = 100; quads[1].pivotY = 100;
			invisibleSprite.addChild ( quads[1] );
			
			quads[2] = new Quad(200, 200, 0x0000FF);
			quads[2].x = stage.stageWidth; quads[2].y = stage.stageHeight;
			//set rotation center to be in the center of the quad
			quads[2].pivotX = 100; quads[2].pivotY = 100;
			invisibleSprite.addChild ( quads[2] );
			
			quads[3] = new Quad(200, 200, 0xAA00FF);
			quads[3].x = 0; quads[3].y = stage.stageHeight;
			//set rotation center to be in the center of the quad
			quads[3].pivotX = 100; quads[3].pivotY = 100;
			invisibleSprite.addChild ( quads[3] );
			
			quads[4] = new Quad(200, 200, 0xAAAAAA);
			quads[4].x = stage.stageWidth *0.5; quads[4].y = stage.stageHeight*0.5;
			//set rotation center to be in the center of the quad
			quads[4].pivotX = 100; quads[4].pivotY = 100;
			addChild ( quads[4] );
			
			infoLabel = new TextField(100, 100, "Tap Quads to start Recording", "Verdana",12, 0xFFFFFF);
			infoLabel.x = stage.stageWidth *0.5 - 50;
			addChild(infoLabel);
			
			this.addEventListener( TouchEvent.TOUCH, onTouch);
			this.addEventListener(EnterFrameEvent.ENTER_FRAME, onEnterFrame);
			
			initializeGameplayRecorder();
		}
		
		// the corresponding event listener
		private function onEnterFrame(event:EnterFrameEvent):void
		{
			m_frameExecTimeStart = getTimer();
			
			
			for ( var quadID:int = 0; quadID < 5; quadID++ ) 
			{ 
				//rotate each quad with 36 degrees per second
				quads[quadID].rotation += 0.0174532925 * 36 * event.passedTime; 
			}
			
			m_frameExecTimeAvg += getTimer() - m_frameExecTimeStart;
			m_frameExecCount++;
		}
		
		private function onTouch(event:TouchEvent):void
		{
			var touch:Touch = event.getTouch(this, TouchPhase.BEGAN);
			if(touch)
			{

				//start recording if we didn't do it already				
				if(!m_isRecording)
				{
					startRecording();
				}
				else
				{
					finishRecording();
				}
				
				trace("StarlingEvent: onTouch");
			}
		}
		
		/* ***************************
		*    ScreenCast ANE calls
		* ***************************/
		private function initializeGameplayRecorder() : void
		{
			m_screenRecorder = new GameplayRecorder();
			m_screenRecorder.addEventListener( GameplayRecorderEvent.VIDEO_SAVED, onVideoSaved );
			m_screenRecorder.addEventListener( GameplayRecorderEvent.ERROR, onScreenCastError );
		}
		
		/* *********************************************************************************
		*    This is where the magic...ahm...the drawing to the frame's BitmapData happens
		* *********************************************************************************/
		private function draw3dCallback( _bmp : BitmapData ):void
		{
			copyAsBitmapData( _bmp, invisibleSprite );
		}
		
		public static function copyAsBitmapData(_bmp : BitmapData, sprite:starling.display.DisplayObject):void 
		{
			if (sprite == null) { return; }

			var context:Context3D = Starling.context;
			var support:RenderSupport = new RenderSupport();
			
			RenderSupport.clear();
			
			support.setOrthographicProjection(0, 0, _bmp.width, _bmp.height);
			
			support.transformMatrix(sprite);
			
			support.pushMatrix();
			support.transformMatrix(sprite);
			sprite.render(support, 1.0);
			support.popMatrix();
			support.finishQuadBatch();
			
			context.drawToBitmapData(_bmp);
		}
		
		private function startRecording() : void
		{
			
			m_callbackExecTimeAvg = 0;
			m_callbackExecCount = 0;
			
			m_frameExecTimeAvg = 0;
			m_frameExecCount = 0;
			
			// 1. Get the path to the file where the video should be saved
			// Make sure any subfolders already exist:
			var subDir : File = File.documentsDirectory.resolvePath( SAVED_VIDEO_SUBFOLDER );
			if ( !subDir.exists )
			{
				subDir.createDirectory();
			}
				
			// 2. ... and the optimal video size, based on the container size and the video codec
			// When using VideoCodecH264 the video width must be one of the standard video widths: 320, 640, etc.
			// When using VideoCodecJpeg both the width and the height of the video must be one of the standard ones.
			var videoCodec : Number = VideoSettings.VideoCodecJpeg; //VideoCodecH264;
			var videoSize : Point = VideoSettings.getStandardVideoSize( stage.stageWidth, stage.stageHeight, videoCodec );
			
			// 3. If the video size isn't the same as the container's, 
			//    decide whether we want the container centered in the video frame (or offset in any way)
			var containerOffsetInFrame : Point = new Point();
			
			// 4. Configure the rest of the video settings
			var videoSettings : VideoSettings = new VideoSettings();
			
			videoSettings.videoCodec = videoCodec; 					// Can be VideoSettings.VideoCodecH264 or VideoCodecJpeg.
			videoSettings.jpegQuality = 0.5; 						// Takes values between 0.0 and 1.0. Specifies JPEG coded quality. Only has effect for videoSettings.videoCodec set to VideoSettings.VideoCodecJpeg.
			videoSettings.h264Quality = VideoSettings.H264Main32; 	// Only has effect for videoSettings.VideoCodec set to VideoSettings.VideoCodec264
			//				Takes the following values:	
			//				VideoSettings.H264Baseline30
			//				VideoSettings.H264Baseline31
			//				VideoSettings.H264Baseline41
			//				VideoSettings.H264Main30 
			//				VideoSettings.H264Main31 
			//				VideoSettings.H264Main32 
			//				VideoSettings.H264Main41
			//				VideoSettings.H264High40  // Supported on iOS 6 and later, defaults to VideoSettings.H264Baseline30 on earlier iOS versions
			//				VideoSettings.H264High41  // Supported on iOS 6 and later, defaults to VideoSettings.H264Baseline30 on earlier iOS versions
			
			videoSettings.framesPerSecond 			= 15;		// Frames per second
			videoSettings.videoAverageBitRate 		= 960000; 	// H264 only, bits per second
			
			// This sets the quality preset to be used if the video is mixed with sound after it's been recorded
			// Use one of the following: VideoSettings.VideoCompositionPresetLowQuality
			//							 VideoSettings.VideoCompositionPresetMediumQuality
			// 							 VideoSettings.VideoCompositionPresetHighestQuality
			videoSettings.videoCompositionQuality = VideoSettings.VideoCompositionPresetHighestQuality;
			
			// Optional audio settings
			//var inputAudioFile : File = File.applicationDirectory.resolvePath( "assets/exam_piece.m4a" );
			//videoSettings.inputAudioFilePath = inputAudioFile.nativePath;
			videoSettings.inputAudioFilePath = null; // Use this, in order to disable mixing of sound
			videoSettings.audioOffsetSeconds = 0;//5;
			videoSettings.loopAudio = false;
			videoSettings.cutOffAudioAtEndOfVideo = true;
			
			// Set saveDebugFrames to true to have every 10th frame saved to CameraRoll - for debugging purposes only
			var saveDebugFrames : Boolean = false;
			
			if ( m_screenRecorder.startRecording( 
				videoSize.x,
				videoSize.y,
				videoSettings,
				draw3dCallback,
				saveDebugFrames ) )
			{
				m_isRecording = true;
				infoLabel.text = "Recording in progress...";
				trace("Recording in progress...");
			}
			else
			{
				infoLabel.text = "ERROR: Could not start recording";
				trace("ERROR: Could not start recording");
			}
		}
		
		private function sleep(ms:int):void {
			var init:int = getTimer();
			while(true) {
				if(getTimer() - init >= ms) {
					break;
				}
			}
		}
		
		private function setPrecision(number:Number, precision:int) 
		{
			precision = Math.pow(10, precision);
			return (Math.round(number * precision)/precision);
		}
		
		private function finishRecording() : void
		{	
			var stats:String = "FD: "+setPrecision(m_frameExecTimeAvg / m_frameExecCount, 4) + "\n";
			stats += "ACB: "+setPrecision(m_callbackExecTimeAvg / m_callbackExecCount, 4) + "\n";
			stats += "ANE: "+ setPrecision(m_screenRecorder.getCallbackExecTime(), 4) + "\n";
			
			trace("Frame Avg: "+(m_frameExecTimeAvg / m_frameExecCount));
			trace("Frame calls: "+m_frameExecCount);
			
			trace("Callback Avg: "+(m_callbackExecTimeAvg / m_callbackExecCount));
			trace("Callback calls: "+m_callbackExecCount);
			
			
			
			infoLabel.text = stats;
			
			var saveVideoToCameraRoll : Boolean = true;
			
			m_screenRecorder.finishRecording( saveVideoToCameraRoll );
			m_isRecording = false;
			
			//infoLabel.text = "Saving video file...";
			trace("Saving video file...");
		}
		
		
		private function onVideoSaved( _event : GameplayRecorderEvent ) : void
		{
			//infoLabel.text = "Video saved";
			trace( "Video saved");
		}
		
		
		private function onScreenCastError( _event : GameplayRecorderEvent ) : void
		{
			// deal with errors from the native extension
			trace( "ScreenCastNativeExtension error: " + _event.message );
		}
		
		private var m_screenRecorder 	: GameplayRecorder = null;
		private var m_savedVideoPath 	: String = null;
		
		private var m_isRecording		: Boolean = false;
		
		private static const SAVED_VIDEO_SUBFOLDER : String = "diadraw/videos";
		private static const SAVED_VIDEO_FILE_NAME : String = "tempVideoFile.mp4";
		
		private var m_callbackExecTimeStart:int = 0;
		private var m_callbackExecCount:int = 0;
		private var m_callbackExecTimeAvg:int = 0;
		
		private var m_frameExecTimeStart:int = 0;
		private var m_frameExecCount:int = 0;
		private var m_frameExecTimeAvg:int = 0;
	}
}